#!/usr/bin/env apl --script

⍝ © 1981-1983 2023-2024 LAAS CNRS, Toulouse, France
⍝ Use of this software is governed by the MIT licence
⍝ Author Bernard Berthomieu, LAAS CNRS, Toulouse, France

)copy TPN
)copy tools
)copy nets-lib

⍝ if file is used as a script, as in e.g. tina.apl -- -sh -g -v ifip83
⍝ then main will call tina/TINA on last arg with options set
⍝ else use apl interactively: )load this file then call whatever you want

∇main;n
  SETUP  ⍝ defaults
  →(2≠⎕ARG ⍳ ⊂'--script')/0
  L∆SCRIPTING←1
  ⍝ sets options from command line
  args←⎕ARG
  dict←'-g' '-sh' '-m' '-s' '-t' '-c' '-h' '-help' '-v'
  a← 1 + args ⍳ ⊂'--'
GO: A←args[a]
  →(1 2 3 4 5 6 7 8 9 10 = dict ⍳ A)/LV,SH,MU,ST,TI,CL,HE,HE,VB,NET
LV: ⍝ -g; enable liveness analysis
  L∆LIVE←0
  a ← a+1
  →GO
SH: ⍝ -sh; enable classes shift
  L∆SHIFT←1
  a ← a+1
  →GO
MU: ⍝ -m; enable multi-enablesness
  L∆MULTI←1
  a ← a+1
  →GO
ST: ⍝ -s n; set boundedness test (1 or 2, default is 0)
  ⍝ n must be 0|1|2
  n←⍎⊃args[a+1]
  →((n<0)∨(n>2))/FAIL
  L∆BOUNDEDNESSTEST←n
  a ← a+2
  →GO
TI: ⍝ -t s; set cpu time increment (non negative int)
  ⍝ n must be non negative
  n←⍎⊃args[a+1]
  →(n<0)/FAIL
  L∆CPU←n
  a ← a+2
  →GO
CL: ⍝ -c n; set number of classes increment (non negative int)
  ⍝ n must be non negative
  n←⍎⊃args[a+1]
  →(n<0)/FAIL
  L∆CLASSES←n
  a ← a+2
  →GO
HE: ⍝ -h | -help; help
  'Tina version ',VERSION,' -- 1981-1985 (core), 2024 (gnu apl port) -- LAAS/CNRS'
  'usage: tina [-g] [-sh] [-m] [-s (0|1|2)] [-t s (s>0)] [-c n (n>0)] [-h | -help] [-v] net'
  'FLAGS         WHAT                                         DEFAULT'
  '-h | -help    this mode                                    '
  'build flags:                                               '
  '-m            state classes with multi-enabledness         without'
  '-sh           shift classes towards origin                 don''t'
  '-g            skip liveness analysis                       don''t'
  'stopping test and limit flags:                             '
  '-s (0|1|2)    stopping test (none|K&M|TPN)                 -s 0'
  '-c n          limit on number of state classes (none if 0) 1000'
  '-t s          limit on cpu time (seconds, none if 0)       300'
  'output format and options flags:                           '
  '-v            verbose mode                                 digest'
  'net:                                                       '
  'net           net name, assumed declared in nets-lib       '
  ⍎')off'
VB: ⍝ -v ; verbose mode
  L∆VERBOSE←1
  a ← a+1
  →GO
NET: ⍝ net
  ⍝ if first char of args[a] is '-' then fail else
  ⍝ if args[a] is a defined function then proceed
  ⍝ else fail (test to be added)
  net ← ⊃(args[a])
  →('-' = 1 ↑ net)/FAIL
  →(a < ⍴args)/FAIL
  →((⍴⎕CR net) > 0)/OK
  'NET ',net, ' UNDEFINED'
  ⍎')off'
OK:
  tina_s net
  →GO
FAIL:
   ⍝ bad command line
   ⎕PW←256
   'BAD COMMAND LINE'
   →HE
∇

main

⍝ )off called in main, to make interactive version work ...
